@@ -16,16 +16,18 @@ package org.rocksdb;
  */
 public abstract class Comparator extends AbstractComparator<Slice> {
 
-  private final long nativeHandle_;
-
   public Comparator(final ComparatorOptions copt) {
-    super();
-    this.nativeHandle_ = createNewComparator0(copt.nativeHandle_);
+    super(copt);
   }
 
   @Override
-  protected final long getNativeHandle() {
-    return nativeHandle_;
+  protected long initializeNative(final long... nativeParameterHandles) {
+    return createNewComparator0(nativeParameterHandles[0]);
+  }
+
+  @Override
+  final ComparatorType getComparatorType() {
+    return ComparatorType.JAVA_COMPARATOR;
   }
 
   private native long createNewComparator0(final long comparatorOptionsHandle);
