@@ -158,6 +158,25 @@ public interface DBOptionsInterface<T extends DBOptionsInterface> {
    */
   T setRateLimiter(RateLimiter rateLimiter);
 
+  /**
+   * Use to track SST files and control their file deletion rate.
+   *
+   * Features:
+   *  - Throttle the deletion rate of the SST files.
+   *  - Keep track the total size of all SST files.
+   *  - Set a maximum allowed space limit for SST files that when reached
+   *    the DB wont do any further flushes or compactions and will set the
+   *    background error.
+   *  - Can be shared between multiple dbs.
+   *
+   *  Limitations:
+   *  - Only track and throttle deletes of SST files in
+   *    first db_path (db_name if db_paths is empty).
+   *
+   * @param sstFileManager The SST File Manager for the db.
+   */
+  T setSstFileManager(SstFileManager sstFileManager);
+
   /**
    * <p>Any internal progress/error information generated by
    * the db will be written to the Logger if it is non-nullptr,
@@ -243,6 +262,8 @@ public interface DBOptionsInterface<T extends DBOptionsInterface> {
    * </p>
    * <p>If set to 0 (default), we will dynamically choose the WAL size limit to
    * be [sum of all write_buffer_size * max_write_buffer_number] * 2</p>
+   * <p>This option takes effect only when there are more than one column family as
+   * otherwise the wal size is dictated by the write_buffer_size.</p>
    * <p>Default: 0</p>
    *
    * @param maxTotalWalSize max total wal size.
@@ -269,7 +290,10 @@ public interface DBOptionsInterface<T extends DBOptionsInterface> {
    * Statistics objects should not be shared between DB instances as
    * it does not use any locks to prevent concurrent updates.</p>
    *
+   * @param statistics The statistics to set
+   *
    * @return the instance of the current object.
+   *
    * @see RocksDB#open(org.rocksdb.Options, String)
    */
   T setStatistics(final Statistics statistics);
@@ -277,7 +301,9 @@ public interface DBOptionsInterface<T extends DBOptionsInterface> {
   /**
    * <p>Returns statistics object.</p>
    *
-   * @return the instance of the statistics object or null if there is no statistics object.
+   * @return the instance of the statistics object or null if there is no
+   * statistics object.
+   *
    * @see #setStatistics(Statistics)
    */
   Statistics statistics();
@@ -446,6 +472,8 @@ public interface DBOptionsInterface<T extends DBOptionsInterface> {
    *
    * @param baseBackgroundCompactions Suggested number of background compaction
    *     jobs
+   *
+   * @deprecated Use {@link #setMaxBackgroundJobs(int)}
    */
   void setBaseBackgroundCompactions(int baseBackgroundCompactions);
 
@@ -485,6 +513,8 @@ public interface DBOptionsInterface<T extends DBOptionsInterface> {
    * @return the maximum number of concurrent background compaction jobs.
    * @see RocksEnv#setBackgroundThreads(int)
    * @see RocksEnv#setBackgroundThreads(int, int)
+   *
+   * @deprecated Use {@link #setMaxBackgroundJobs(int)}
    */
   int maxBackgroundCompactions();
 
@@ -522,6 +552,8 @@ public interface DBOptionsInterface<T extends DBOptionsInterface> {
    * @see RocksEnv#setBackgroundThreads(int)
    * @see RocksEnv#setBackgroundThreads(int, int)
    * @see #maxBackgroundCompactions()
+   *
+   * @deprecated Use {@link #setMaxBackgroundJobs(int)}
    */
   T setMaxBackgroundFlushes(int maxBackgroundFlushes);
 
@@ -537,6 +569,25 @@ public interface DBOptionsInterface<T extends DBOptionsInterface> {
    */
   int maxBackgroundFlushes();
 
+  /**
+   * Specifies the maximum number of concurrent background jobs (both flushes
+   * and compactions combined).
+   * Default: 2
+   *
+   * @param maxBackgroundJobs number of max concurrent background jobs
+   * @return the instance of the current object.
+   */
+  T setMaxBackgroundJobs(int maxBackgroundJobs);
+
+  /**
+   * Returns the maximum number of concurrent background jobs (both flushes
+   * and compactions combined).
+   * Default: 2
+   *
+   * @return the maximum number of concurrent background jobs.
+   */
+  int maxBackgroundJobs();
+
   /**
    * Specifies the maximum size of a info log file. If the current log file
    * is larger than `max_log_file_size`, a new info log file will
